﻿using System.Linq;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public class ChoiceRange : SimulatedPlaytraceMetric
    {
        int numChoicesSeen;

        int numActionTurns;

        public ChoiceRange(IGraphStore store) : base(store)
        {
        }

        protected override void OnTreeEdge(InteractionEdge edge)
        {
            if (edge.Source.NodeType == InteractionNode.Type.InteractionPoint)
            {
                // This is an action if it was taken from an interaction point.
                numActionTurns++;

                // Count the number of choices.
                numChoicesSeen += GetNumChoices(edge.Source);
            }
        }

        private int GetNumChoices(InteractionNode node)
        {
            return CurrentGraph.Graph.OutEdges(node).Count();
        }

        protected override void OnStartPlaythrough(int playthroughNum)
        {
            numActionTurns = 0;
            numChoicesSeen = 0;
        }

        protected override void OnCompletePlaythrough(int playthroughNum)
        {
            if (numActionTurns == 0)
            {
                AccumulateMetric(0);
                return;
            }

            AccumulateMetric(numChoicesSeen / (double)numActionTurns);
        }

        private void AccumulateMetric(double thisLength)
        {
            Result += thisLength / (double)NumPlaythroughs;
        }
    }
}
