﻿
using System.Collections.Generic;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;
using UnityEngine;

namespace StudyCrafter.Assistant.Metrics
{
    public class ChoiceVariability : SimulatedPlaytraceMetric
    {
        int numActionTurns = 0;
        double uniqueChoicesAtEachPoint = 0;

        HashSet<InteractionEdge> allSeenChoices = new HashSet<InteractionEdge>();

        public ChoiceVariability(IGraphStore store) : base(store)
        {
        }

        protected override void OnTreeEdge(InteractionEdge edge)
        {
            if (edge.Source.NodeType == InteractionNode.Type.InteractionPoint)
            {
                numActionTurns++;

                HashSet<InteractionEdge> choicesThisPoint = new HashSet<InteractionEdge>(GetChoices(edge.Source));
                int totalChoicesThisPoint = choicesThisPoint.Count;

                // Find the intersection with all choices to get previously-seen choices.
                // TODO: Is there a better method for determining whether choices are new? Maybe choices with similar text should not be considered new?
                choicesThisPoint.IntersectWith(allSeenChoices);

                // Unique choices are 1 - the ratio of previously seen vs. total choices available at this point.
                uniqueChoicesAtEachPoint += 1.0 - (choicesThisPoint.Count / totalChoicesThisPoint);

                // Add all choices this point to the set of all seen choices.
                allSeenChoices.UnionWith(GetChoices(edge.Source));
            }
        }

        private IEnumerable<InteractionEdge> GetChoices(InteractionNode node)
        {
            return CurrentGraph.Graph.OutEdges(node);
        }

        protected override void OnStartPlaythrough(int playthroughNum)
        {
            numActionTurns = 0;
            uniqueChoicesAtEachPoint = 0;

            allSeenChoices.Clear();
        }

        protected override void OnCompletePlaythrough(int playthroughNum)
        {
            if (numActionTurns == 0)
            {
                AccumulateMetric(0);
                return;
            }

            AccumulateMetric(uniqueChoicesAtEachPoint / (double)numActionTurns);
        }

        private void AccumulateMetric(double metricValue)
        {
            Result += metricValue / (double)NumPlaythroughs;
        }
    }
}
