﻿using System;
using System.Collections.Generic;
using QuickGraph;
using QuickGraph.Algorithms.ConnectedComponents;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public class NumConnectedComponents : ScriptMetric
    {
        public NumConnectedComponents(IGraphStore store) : base(store)
        {
        }

        protected override void CalculateForGraph(ScriptGraph graph)
        {
            StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>> alg
                = new StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>>(graph.Graph);
            alg.Finished += OnCalculationFinished;
            alg.Compute();
        }

        private void OnCalculationFinished(object sender, EventArgs result)
        {
            StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>> algorithm
                = (StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>>)sender;

            AccumulateMetric(FindNumNonTrivialComponents(algorithm));
        }

        private Dictionary<int, int> AccumulateSizes(StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>> algorithm)
        {
            Dictionary<int, int> componentSizes = new Dictionary<int, int>();
            foreach (var component in algorithm.Components)
            {
                if (!componentSizes.ContainsKey(component.Value))
                {
                    componentSizes[component.Value] = 0;
                }
                componentSizes[component.Value]++;
            }

            return componentSizes;
        }

        private int FindNumNonTrivialComponents(StronglyConnectedComponentsAlgorithm<ScriptNode, IEdge<ScriptNode>> algorithm)
        {
            Dictionary<int, int> componentSizes = AccumulateSizes(algorithm);

            int numNonTrivialComponents = 0;
            foreach (KeyValuePair<int, int> size in componentSizes)
            {
                if (size.Value > 1)
                {
                    numNonTrivialComponents++;
                }
            }

            return numNonTrivialComponents;
        }
    }
}
