﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using QuickGraph;
using UnityEngine;

namespace StudyCrafter.Assistant.Representation
{
	public class GraphStore : IGraphStore
	{
		private ServiceContainer services = new ServiceContainer();

		public bool AddOwner<TVertex, TEdge>(IGraphOwner<TVertex, TEdge> graphOwner)
			where TVertex : IGraphNode
			where TEdge : IEdge<TVertex>
		{
			GraphCollection<TVertex, TEdge> collection = services.GetService(typeof(GraphCollection<TVertex, TEdge>)) as GraphCollection<TVertex, TEdge>;
			if (null == collection)
			{
				collection = new GraphCollection<TVertex, TEdge>();
				services.AddService(collection.GetType(), collection);
			}

			if (collection.Contains(graphOwner.ID))
			{
				return false;
			}

			collection.AddOwner(graphOwner);

			return true;
		}

		public bool RemoveOwner<TVertex, TEdge>(GraphID id)
			where TVertex : IGraphNode
			where TEdge : IEdge<TVertex>
		{
			GraphCollection<TVertex, TEdge> collection = services.GetService(typeof(GraphCollection<TVertex, TEdge>)) as GraphCollection<TVertex, TEdge>;
			if (null == collection)
			{
				return false;
			}

			return collection.RemoveOwner(id);
		}

		public bool TryGetOwner<TVertex, TEdge>(GraphID id, out IGraphOwner<TVertex, TEdge> graphOwner)
			where TVertex : IGraphNode
			where TEdge : IEdge<TVertex>
		{
			GraphCollection<TVertex, TEdge> collection = services.GetService(typeof(GraphCollection<TVertex, TEdge>)) as GraphCollection<TVertex, TEdge>;
			if (null == collection)
			{
				graphOwner = null;
				return false;
			}

			return collection.TryGetOwner(id, out graphOwner);
		}
	}
}
