﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using StudyCrafter.Assistant.StudyCrafterTypes;

namespace StudyCrafter.Assistant.Representation
{
	public class ConditionalProperty : IProperty
	{
		public class Branch
		{
			public IntProperty Index { get; set; }
			public ExpressionProperty Condition { get; set; }
		}

		private static Regex branchesRegex;

		static ConditionalProperty()
		{
			// Determine the gap strings use.
			IfStateNode ifNode = new IfStateNode();
			branchesRegex = new Regex(String.Format(@"(\d+?){0}(.*?){1}", ifNode.smallGapString, ifNode.largeGapString));
		}

		public ConditionalProperty(string id, string val)
		{
			ID = id;
			Val = val;
		}

		public string ID
		{
			get;
			private set;
		}

		public string Val
		{
			get
			{
				return BranchesString;
			}
			private set
			{
				BranchesString = value;
			}
		}

		private string branchesString;
		public string BranchesString
		{
			get
			{
				return branchesString;
			}
			private set
			{
				branchesString = value;
				Parse();
			}
		}

		public List<Branch> Branches
		{
			get;
			private set;
		}

		private void Parse()
		{
			List<Branch> branches = new List<Branch>();

			MatchCollection matches = branchesRegex.Matches(BranchesString);
			foreach (Match match in matches)
			{
				Branch branch = new Branch()
				{
					Index = new IntProperty("index", match.Groups[1].Value),
					Condition = new ExpressionProperty("condition", match.Groups[2].Value),
				};
				branches.Add(branch);
			}

			Branches = branches;
		}
	}
}

