﻿using System.Collections.Generic;

namespace StudyCrafter.Assistant.Representation
{
    public class PropertyDictionary : IPropertyDictionary
    {
        private IDictionary<string, IProperty> properties = new Dictionary<string, IProperty>();

        public PropertyDictionary()
        {
        }

        public IProperty Get(string name)
        {
            return properties[name];
        }

        public bool TryGet(string name, out IProperty property)
        {
            return properties.TryGetValue(name, out property);
        }

        public void Set(IProperty val)
        {
            properties[val.ID] = val;
        }

        public bool Remove(string name)
        {
            return properties.Remove(name);
        }

        public T Get<T>(string name) where T : IProperty
        {
            IProperty property = Get(name);
            if (property is T)
            {
                return (T)property;
            }
            return default(T);
        }

        public bool TryGet<T>(string name, out T val)
        {
            IProperty property;
            if (!TryGet(name, out property))
            {
                val = default(T);
                return false;
            }

            if (property is T)
            {
                val = (T)property;
                return true;
            }

            val = default(T);
            return false;
        }

        public void Set<T>(T val) where T : IProperty
        {
            Set((IProperty)val);
        }

        public bool Remove<T>(string name) where T : IProperty
        {
            return Remove(name);
        }

        public bool Contains(string name)
        {
            return properties.ContainsKey(name);
        }

        public IEnumerable<IProperty> All()
        {
            return properties.Values;
        }

        public void AddAll(IPropertyDictionary other)
        {
            foreach (IProperty property in other.All())
            {
                Set(property);
            }
        }

    }
}
