﻿using System;

namespace StudyCrafter.Assistant.Representation
{
    public static class PropertyFactory
    {
        public static IProperty OfType(PropertyType type, string name, string propertyValue)
        {
            switch (type)
            {
                case PropertyType.String:
                    {
                        return new StringProperty(name, propertyValue);
                    }
                case PropertyType.Int:
                    {
                        return new IntProperty(name, propertyValue);
                    }
                case PropertyType.Double:
                    {
                        return new DoubleProperty(name, propertyValue);
                    }
                case PropertyType.Bool:
                    {
                        return new BoolProperty(name, propertyValue);
                    }
                case PropertyType.Expression:
                    {
                        return new ExpressionProperty(name, propertyValue);
                    }
                case PropertyType.Variable:
                    {
                        return new VariableProperty(name, propertyValue);
                    }
                case PropertyType.Choices:
                    {
                        return new ChoicesProperty(name, propertyValue);
                    }
                case PropertyType.Conditional:
                    {
                        return new ConditionalProperty(name, propertyValue);
                    }
                case PropertyType.LayoutObject:
                    {
                        return new LayoutObjectProperty(name, propertyValue);
                    }
                case PropertyType.StringExpression:
                    {
                        return new StringExpressionProperty(name, propertyValue);
                    }
                default:
                    {
                        throw new InvalidOperationException("Error: no factory method set up for property type " + type);
                    }
            }
        }
    }
}
