﻿using System.Text.RegularExpressions;
using UnityEngine;

namespace StudyCrafter.Assistant.Representation
{
    public class StringExpressionProperty : IProperty
    {
        // Match {, then anything but } repeated any number of times, then }.
        private static Regex varsRegex = new Regex(@"{([^}]+)}");

        private IPropertyDictionary variables;

        private string fullString;

        public StringExpressionProperty(string id, string val)
        {
            ID = id;
            Val = val;
        }

        public string ID
        {
            get;
            private set;
        }

        public string Val
        {
            get
            {
                return fullString;
            }
            private set
            {
                fullString = value;

                if (null != fullString)
                {
                    ParseVars();
                }
            }
        }

        private void ParseVars()
        {
            Variables = new PropertyDictionary();

            int index = 0;
            MatchCollection matches = varsRegex.Matches(fullString);
            foreach (Match match in matches)
            {
                string varName = match.Groups[1].Value;
                VariableProperty varProperty = new VariableProperty(index.ToString(), varName);
                Variables.Set(varProperty);
                ++index;
            }
        }

        public IPropertyDictionary Variables
        {
            get
            {
                return variables;
            }
            private set
            {
                variables = value;
            }
        }
    }
}