﻿
using System.Collections.Generic;
using Newtonsoft.Json;
using QuickGraph;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.StudyCrafterLoading
{
    public static class SceneGraphJsonLoadingExtensions
    {
        public static SceneGraph AddStartScene(this SceneGraph graph, string rootJsonFile)
        {
            ScenarioMetadata scenarioMetadata = JsonConvert.DeserializeObject<ScenarioMetadata>(rootJsonFile);

            graph.Root = new Scene(scenarioMetadata.StartScene);

            return graph;
        }

        public static SceneGraph BuildSceneGraph(this SceneGraph graph, IDictionary<string, string> sceneJsonFiles, StudyCrafterLoadHelpers loadHelpers)
        {
            IDictionary<string, SceneSerializationData> sceneDataDict = new Dictionary<string, SceneSerializationData>();
            foreach (KeyValuePair<string, string> sceneFile in sceneJsonFiles)
            {
                SceneSerializationData sceneData = JsonConvert.DeserializeObject<SceneSerializationData>(sceneFile.Value);
                sceneDataDict[sceneData.Key] = sceneData;
            }

            foreach (KeyValuePair<string, SceneSerializationData> sceneData in sceneDataDict)
            {
                graph.BuildScene(sceneData.Value, loadHelpers);
            }

            return graph;
        }

        public static SceneGraph BuildScene(this SceneGraph graph, SceneSerializationData sceneData, StudyCrafterLoadHelpers loadHelpers)
        {
            Scene scene = graph.GetOrAddVertex(new Scene(sceneData.Key))
                                .FromSerializationData(sceneData, loadHelpers);

            return graph.AddSceneTransitions(scene, sceneData);
        }

        public static SceneGraph AddSceneTransitions(this SceneGraph graph, Scene scene, SceneSerializationData sceneData)
        {
            foreach (string otherScene in sceneData.GetAllSceneTransitions())
            {
                Scene otherSceneNode = graph.GetOrAddVertex(new Scene(otherScene));
                graph.Graph.AddEdge(new Edge<Scene>(scene, otherSceneNode));
            }
            return graph;
        }

        public static IEnumerable<string> GetAllSceneTransitions(this SceneSerializationData scene)
        {
            HashSet<string> result = new HashSet<string>();

            foreach (ScriptNodeSerializationData scriptNode in scene.Scripts.Values)
            {
                if (scriptNode.Type == "end")
                {
                    string nextScene = scriptNode.GetProperty("nextScript");
                    if (!string.IsNullOrEmpty(nextScene) && nextScene != "None")
                    {
                        result.Add(nextScene);
                    }
                }
            }

            return result;
        }

    }
}
