﻿using System;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;
using UnityEngine;

namespace StudyCrafter.Assistant.StudyCrafterLoading
{
    public static class ScriptNodeJsonLoadingExtensions
    {
        public static ScriptNode FromSerializationData(this ScriptNode scriptNode, ScriptNodeSerializationData scriptData, ScriptNodeDefCollection nodeDefs)
        {
            ScriptNodeDefinition nodeDef;
            if (!nodeDefs.TryGetDefinition(scriptData.Type, out nodeDef))
            {
                throw new InvalidOperationException("Error: no definition specified for script node type " + scriptData.Type);
            }

            scriptNode.Type = scriptData.Type;

            scriptNode.FillProperties(scriptData, nodeDef);

            return scriptNode;
        }

        public static ScriptNode FillProperties(this ScriptNode scriptNode, ScriptNodeSerializationData scriptData, ScriptNodeDefinition nodeDef)
        {
            foreach (PropertyDefinition propertyDef in nodeDef.Properties)
            {
                string propertyValue;
                if (scriptData.TryGetProperty(propertyDef.Name, out propertyValue))
                {
                    try
                    {
                        IProperty instantiatedProperty = propertyDef.PropertyWithValue(propertyValue);
                        scriptNode.SetProperty(instantiatedProperty);
                    }
                    catch (Exception e)
                    {
                        Debug.LogError("Exception while reading " + propertyDef.Name + " of type " + propertyDef.Type + " in script node " + scriptNode.ID);
                        throw e;
                    }
                }
            }

            CheckAllPropertiesFound(scriptNode, scriptData);

            return scriptNode;
        }

        private static bool CheckAllPropertiesFound(ScriptNode scriptNode, ScriptNodeSerializationData scriptData)
        {
            bool allFound = true;
            // Check that all properties were read. If not, warn that there's an unknown property.
            foreach (string propName in scriptData.GetPropertyNames())
            {
                if (!scriptNode.HasProperty(propName))
                {
                    Debug.LogWarning("Script Node: " + scriptNode.ID + " of type: " + scriptNode.Type + " - Unread property: " + propName);
                    allFound = false;
                }
            }

            return allFound;
        }
    }
}