﻿using System.Collections;
using System.Collections.Generic;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.StudyCrafterLoading
{
	public class ScriptToInteractionMapping : IDictionary<ScriptNode, InteractionMapSubgraph>
	{
		private IDictionary<ScriptNode, InteractionMapSubgraph> storageDict = new Dictionary<ScriptNode, InteractionMapSubgraph>();

		public ICollection<ScriptNode> Keys
		{
			get
			{
				return storageDict.Keys;
			}
		}

		public ICollection<InteractionMapSubgraph> Values
		{
			get
			{
				return storageDict.Values;
			}
		}

		public int Count
		{
			get
			{
				return storageDict.Count;
			}
		}

		public bool IsReadOnly
		{
			get
			{
				return storageDict.IsReadOnly;
			}
		}

		public InteractionMapSubgraph this[ScriptNode key]
		{
			get
			{
				return storageDict[key];
			}

			set
			{
				storageDict[key] = value;
			}
		}

		public ScriptToInteractionMapping()
		{
		}

		public void Add(ScriptNode key, InteractionMapSubgraph value)
		{
			storageDict.Add(key, value);
		}

		public bool ContainsKey(ScriptNode key)
		{
			return storageDict.ContainsKey(key);
		}

		public bool Remove(ScriptNode key)
		{
			return storageDict.Remove(key);
		}

		public bool TryGetValue(ScriptNode key, out InteractionMapSubgraph value)
		{
			return storageDict.TryGetValue(key, out value);
		}

		public void Add(KeyValuePair<ScriptNode, InteractionMapSubgraph> item)
		{
			storageDict.Add(item);
		}

		public void Clear()
		{
			storageDict.Clear();
		}

		public bool Contains(KeyValuePair<ScriptNode, InteractionMapSubgraph> item)
		{
			return storageDict.Contains(item);
		}

		public void CopyTo(KeyValuePair<ScriptNode, InteractionMapSubgraph>[] array, int arrayIndex)
		{
			storageDict.CopyTo(array, arrayIndex);
		}

		public bool Remove(KeyValuePair<ScriptNode, InteractionMapSubgraph> item)
		{
			return storageDict.Remove(item);
		}

		public IEnumerator<KeyValuePair<ScriptNode, InteractionMapSubgraph>> GetEnumerator()
		{
			return storageDict.GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return storageDict.GetEnumerator();
		}

		public void RegisterEntryPoint(ScriptNode scriptNode, InteractionNode interactionNode)
		{
			InteractionMapSubgraph subgraph;
			if (!storageDict.TryGetValue(scriptNode, out subgraph))
			{
				subgraph = new InteractionMapSubgraph();
				storageDict.Add(scriptNode, subgraph);
			}

			subgraph.EntryNode = interactionNode;
		}

		public void RegisterExitPoint(ScriptNode scriptNode, InteractionNode interactionNode)
		{
			InteractionMapSubgraph subgraph;
			if (!storageDict.TryGetValue(scriptNode, out subgraph))
			{
				subgraph = new InteractionMapSubgraph();
				storageDict.Add(scriptNode, subgraph);
			}

			subgraph.ExitNode = interactionNode;
		}

		public void RegisterSingleNodeSubgraph(ScriptNode scriptNode, InteractionNode interactionNode)
		{
			InteractionMapSubgraph subgraph;
			if (!storageDict.TryGetValue(scriptNode, out subgraph))
			{
				subgraph = new InteractionMapSubgraph();
				storageDict.Add(scriptNode, subgraph);
			}

			subgraph.EntryNode = interactionNode;
			subgraph.ExitNode = interactionNode;
		}

		public bool TryGetExitPoint(ScriptNode scriptNode, out InteractionNode interactionNode)
		{
			InteractionMapSubgraph subgraph;
			if (!storageDict.TryGetValue(scriptNode, out subgraph))
			{
				interactionNode = null;
				return false;
			}

			interactionNode = subgraph.ExitNode;
			return true;
		}

		public bool TryGetEntryPoint(ScriptNode scriptNode, out InteractionNode interactionNode)
		{
			InteractionMapSubgraph subgraph;
			if (!storageDict.TryGetValue(scriptNode, out subgraph))
			{
				interactionNode = null;
				return false;
			}

			interactionNode = subgraph.EntryNode;
			return true;
		}
	}
}

