﻿using System;
using System.Collections;
using System.Collections.Generic;
using StudyCrafter.Assistant.Loading;
using UnityEngine;

namespace StudyCrafter.Assistant.StudyCrafterLoading
{
    public class StreamingAssetsFileLoader : MonoBehaviour
    {
        private class LoadData
        {
            public LoadStatus Status
            {
                get;
                private set;
            }

            public string Result
            {
                get;
                set;
            }

            public LoadData()
            {
                Status = LoadStatus.None;
                Result = String.Empty;
            }

            public void UpdateStatus(LoadStatus status)
            {
                Status = status;
            }

            public void OnResult(string result)
            {
                Result = result;
                Status = LoadStatus.Loaded;
            }

        }

        private IDictionary<string, LoadData> results = new Dictionary<string, LoadData>();

        public StreamingAssetsFileLoader()
        {

        }

        public string RetrieveResult(string fileName)
        {
            LoadData data;
            if (!results.TryGetValue(fileName, out data))
            {
                return null;
            }

            if (data.Status != LoadStatus.Loaded)
            {
                return null;
            }

            results.Remove(fileName);

            return data.Result;
        }

        public LoadStatus GetStatus(string fileName)
        {
            LoadData data;
            if (!results.TryGetValue(fileName, out data))
            {
                data = new LoadData();
                results.Add(fileName, data);
            }

            return data.Status;
        }

        public LoadStatus Load(string fileName)
        {
            LoadData data;
            if (!results.TryGetValue(fileName, out data))
            {
                data = new LoadData();
                results.Add(fileName, data);
            }

            StartCoroutine(LoadCoroutine(fileName));

            return data.Status;
        }

        public IEnumerator LoadCoroutine(string fileName)
        {
            results[fileName].UpdateStatus(LoadStatus.InProgress);

            string filePath = System.IO.Path.Combine(Application.streamingAssetsPath, fileName);
            string result = "";
            if (filePath.Contains("://"))
            {
                UnityEngine.Networking.UnityWebRequest www = UnityEngine.Networking.UnityWebRequest.Get(filePath);
                yield return www.SendWebRequest();
                result = www.downloadHandler.text;
            }
            else
            {
                result = System.IO.File.ReadAllText(filePath);
            }

            results[fileName].OnResult(result);
        }
    }
}
