﻿using System.IO;
using QuickGraph;
using QuickGraph.Algorithms;
using QuickGraph.Algorithms.Services;
using QuickGraph.Graphviz;
using StudyCrafter.Assistant.Representation;

namespace StudyCrafter.Assistant.StudyCrafterSaving
{
    public class GraphToDotExporter<TGraphOwner, TVertex, TEdge> : AlgorithmBase<TGraphOwner> where TGraphOwner : IGraphOwner<TVertex, TEdge> where TEdge : IEdge<TVertex>
    {
        public string FilePath { get; private set; }

        public string FileNamePrefix { get; private set; }

        public GraphToDotExporter(string filePath, string fileNamePrefix, TGraphOwner visitedGraph) : base(visitedGraph)
        {
            FilePath = filePath;
            FileNamePrefix = fileNamePrefix;
        }

        public GraphToDotExporter(string filePath, string fileNamePrefix, IAlgorithmComponent host, TGraphOwner visitedGraph) : base(host, visitedGraph)
        {
            FilePath = filePath;
            FileNamePrefix = fileNamePrefix;
        }

        protected override void InternalCompute()
        {
            GraphvizAlgorithm<TVertex, TEdge> visAlg = new GraphvizAlgorithm<TVertex, TEdge>(VisitedGraph.Graph);
            ModifyVisAlg(visAlg);
            string dotSerializedGraph = visAlg.Generate();

            WriteGraph(dotSerializedGraph);
        }

        private void WriteGraph(string graphString)
        {
            string fileName = string.Format("{0}-{1}-{2}.dot", FileNamePrefix, typeof(TGraphOwner).Name, VisitedGraph.ID);
            var outfile = File.CreateText(Path.Combine(FilePath, fileName));
            outfile.WriteLine(graphString);
            outfile.Close();
        }

        protected virtual void ModifyVisAlg(GraphvizAlgorithm<TVertex, TEdge> visAlg)
        {
            visAlg.FormatVertex += FormatVertex;
            visAlg.FormatEdge += FormatEdge;
        }

        protected virtual void FormatVertex(object sender, FormatVertexEventArgs<TVertex> args)
        {
            // do nothing.
        }

        protected virtual void FormatEdge(object sender, FormatEdgeEventArgs<TVertex, TEdge> args)
        {
            // do nothing.
        }
    }
}
