﻿using System.Linq;
using System.Text;
using QuickGraph.Algorithms.Services;
using QuickGraph.Graphviz.Dot;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.StudyCrafterSaving
{
	public class InteractionMapToDotExporter : GraphToDotExporter<InteractionMap, InteractionNode, InteractionEdge>
	{
		public InteractionMapToDotExporter(string filePath, string fileNamePrefix, InteractionMap visitedGraph) : base(filePath, fileNamePrefix, visitedGraph)
		{
		}

		public InteractionMapToDotExporter(string filePath, string fileNamePrefix, IAlgorithmComponent host, InteractionMap visitedGraph) : base(filePath, fileNamePrefix, host, visitedGraph)
		{
		}

		protected override void FormatVertex(object sender, QuickGraph.Graphviz.FormatVertexEventArgs<InteractionNode> args)
		{
			base.FormatVertex(sender, args);

			args.VertexFormatter.Shape = GraphvizVertexShape.Rectangle;

			args.VertexFormatter.Label = string.Format("<{0} <BR/> <FONT POINT-SIZE=\"9\"> {1} </FONT> >",
													   args.VertexFormatter.Label,
													   FormatExtraInfo(args.Vertex));
		}

		protected string FormatExtraInfo(InteractionNode node)
		{
			StringBuilder sb = new StringBuilder();
			if (node.InteractionUnits.Count() > 0)
			{
				sb.Append(string.Format("Units: {0}", string.Join(",", node.InteractionUnits.ToArray())));
			}

			StringProperty scriptID;
			if (node.TryGetProperty<StringProperty>("start_script_id", out scriptID))
			{
				sb.Append(string.Format("<BR/> Script: {0} - {1}", scriptID.Val, node.GetProperty<StringProperty>("script_type").Val));
			}

			return sb.ToString();
		}

		protected override void FormatEdge(object sender, QuickGraph.Graphviz.FormatEdgeEventArgs<InteractionNode, InteractionEdge> args)
		{
			base.FormatEdge(sender, args);

			if (args.Edge.EdgeType == InteractionEdge.Type.Indirect)
			{
				args.EdgeFormatter.Style = QuickGraph.Graphviz.Dot.GraphvizEdgeStyle.Dotted;
			}
			else if (args.Edge.EdgeType == InteractionEdge.Type.Choice)
			{
				args.EdgeFormatter.Style = QuickGraph.Graphviz.Dot.GraphvizEdgeStyle.Dashed;
			}
		}
	}
}
