﻿
using System;
using QuickGraph;

namespace StudyCrafter.Assistant.StudyCrafterScenario
{
    public class InteractionEdge : Edge<InteractionNode>, IEquatable<InteractionEdge>
    {
        public enum Type
        {
            Choice,
            Direct,
            Indirect
        }

        public InteractionEdge.Type EdgeType
        {
            get; private set;
        }

		public int ID
		{
			get; private set;
		}

        public InteractionEdge(InteractionEdge other) : base(other.Source, other.Target)
        {
            EdgeType = other.EdgeType;
			ID = other.ID;
        }

		public InteractionEdge(InteractionEdge.Type type, InteractionNode source, InteractionNode target) : base(source, target)
		{
			EdgeType = type;
		}

        public InteractionEdge(InteractionEdge.Type type, InteractionNode source, InteractionNode target, int id) : base(source, target)
        {
            EdgeType = type;
			ID = id;
        }

		public void SetID(int id)
		{
			ID = id;
		}

        public bool Equals(InteractionEdge other)
        {
            if (object.ReferenceEquals(other, null))
            {
                return false;
            }

			return
				this.ID == other.ID &&
                this.Source.Equals(other.Source) &&
                this.Target.Equals(other.Target);
        }

        public override bool Equals(object obj)
        {
            return this.Equals(obj as InteractionEdge);
        }

        public override int GetHashCode()
        {
            return HashCodeHelper.Combine(
                this.Source.GetHashCode(),
                this.Target.GetHashCode(),
				this.ID);
        }

        public static bool operator ==(InteractionEdge first, InteractionEdge second)
        {
            if (object.ReferenceEquals(first, null))
            {
                return object.ReferenceEquals(second, null);
            }

            return first.Equals(second);
        }


        public static bool operator !=(InteractionEdge first, InteractionEdge second)
        {
            if (object.ReferenceEquals(first, null))
            {
                return object.ReferenceEquals(second, null);
            }

            return !first.Equals(second);
        }
    }
}
