﻿using System;
using UnityEngine;
using UnityEngine.UI;

namespace StudyCrafter.Assistant.UI
{
	public class AnalyzeProjectController : MonoBehaviour
	{
		[SerializeField]
		private ProjectAnalysisComponent analyzer = null;

		[SerializeField]
		private Button button;

		[SerializeField]
		private StatusTextController statusText;

		[SerializeField]
		private PathController representationExportPathController;

		[SerializeField]
		private PathController metricsExportPathController;

		[SerializeField]
		private PathController projectsDirectoryPathController;

		private string projectName;

		public AnalyzeProjectController()
		{
		}

		public void SetProject(string project)
		{
			projectName = project;
		}

		public void Start()
		{
			if (null != analyzer)
			{
				analyzer.FinishedEventHandler += OnAnalysisFinished;
				analyzer.ErrorEventHandler += OnAnalysisError;
			}

			if (null != representationExportPathController)
			{
				if (null != analyzer)
				{
					representationExportPathController.FilePath = analyzer.RepresentationExportPath;
				}
				representationExportPathController.PathUpdatedListener += OnRepresentationExportPathUpdated;
			}

			if (null != metricsExportPathController)
			{
				if (null != analyzer)
				{
					metricsExportPathController.FilePath = analyzer.MetricsExportPath;
				}
				metricsExportPathController.PathUpdatedListener += OnMetricsExportPathUpdated;
			}

			if (null != projectsDirectoryPathController)
			{
				if (null != analyzer)
				{
					projectsDirectoryPathController.FilePath = analyzer.ProjectsDirectoryPath;
				}
				projectsDirectoryPathController.PathUpdatedListener += OnProjectsDirectoryPathUpdated;
			}
		}

		public void OnClick()
		{
			if (null == analyzer)
			{
				OnAnalysisError("Project analysis controller has no associated analysis component!");
				return;
			}

			if (string.IsNullOrEmpty(projectName))
			{
				OnAnalysisError("No project name specified!");
				return;
			}

			button.enabled = false;

			statusText.DisplayMessage("Analyzing project: " + projectName + " ...");
			analyzer.SetProject(projectName);
			analyzer.LoadAndAnalyze();
		}

		void OnAnalysisFinished()
		{
			statusText.DisplayMessage("Finished!");

			button.enabled = true;
		}

		void OnAnalysisError(string error)
		{
			statusText.DisplayError(error);

			button.enabled = true;
		}

		void OnRepresentationExportPathUpdated(string exportPath)
		{
			analyzer.RepresentationExportPath = exportPath;
		}

		void OnMetricsExportPathUpdated(string exportPath)
		{
			analyzer.MetricsExportPath = exportPath;
		}

		void OnProjectsDirectoryPathUpdated(string projectsPath)
		{
			analyzer.ProjectsDirectoryPath = projectsPath;
		}
	}
}

